﻿using RimWorld;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Verse;

namespace DraftAnything {
    static class InjectedMethods {
        public static void AddAndRemoveDynamicComponents(Pawn pawn, bool actAsIfSpawned = false) {
            Log.Message("Injected AddAndRemoveDynamicComponents called for pawn: " + pawn.NameStringShort);

            bool flag = pawn.Faction != null && pawn.Faction.IsPlayer;
            bool flag2 = pawn.HostFaction != null && pawn.HostFaction.IsPlayer;
            if (pawn.RaceProps.Humanlike && !pawn.Dead) {
                if (pawn.mindState.wantsToTradeWithColony) {
                    if (pawn.trader == null) {
                        pawn.trader = new Pawn_TraderTracker(pawn);
                    }
                } else {
                    pawn.trader = null;
                }
            }
            if (pawn.RaceProps.Humanlike) {
                if (flag) {
                    if (pawn.outfits == null) {
                        pawn.outfits = new Pawn_OutfitTracker(pawn);
                    }
                    if (pawn.drugs == null) {
                        pawn.drugs = new Pawn_DrugPolicyTracker(pawn);
                    }
                    if (pawn.timetable == null) {
                        pawn.timetable = new Pawn_TimetableTracker();
                    }
                }
            }
            if (flag || flag2) {
                if (pawn.playerSettings == null) {
                    pawn.playerSettings = new Pawn_PlayerSettings(pawn);
                }
            }
            if (pawn.RaceProps.intelligence <= Intelligence.ToolUser && pawn.Faction != null && !pawn.RaceProps.IsMechanoid && pawn.training == null) {
                pawn.training = new Pawn_TrainingTracker(pawn);
            }
            if (pawn.needs != null) {
                pawn.needs.AddOrRemoveNeedsAsAppropriate();
            }

            if ((pawn.Spawned || actAsIfSpawned) && pawn.drafter == null) {
                Log.Message("Drafter added for pawn: " + pawn.NameStringShort);
                pawn.drafter = new Pawn_DraftController(pawn);
            }
        }

        public static IEnumerable<Gizmo> GetGizmos(this Pawn pawn) {
            Log.Message("Injected GetGizmos called for pawn: " + pawn.NameStringShort);

            if (pawn.drafter == null) {
                Log.Message("Drafter added for pawn: " + pawn.NameStringShort);
                pawn.drafter = new Pawn_DraftController(pawn);
            }

            if (pawn.IsColonistPlayerControlled) {
                //foreach (var gizmo in pawn.InvokeBaseMethod<IEnumerable<Gizmo>>("GetGizmos")) { 
                //    yield return gizmo;
                //}

                if (pawn.equipment != null) {
                    foreach (var gizmo in pawn.equipment.GetGizmos()) {
                        yield return gizmo;
                    }
                }

                if (pawn.apparel != null) {
                    foreach (var gizmo in pawn.apparel.GetGizmos()) {
                        yield return gizmo;
                    }
                }

                if (pawn.playerSettings != null) {
                    foreach (var gizmo in pawn.playerSettings.GetGizmos()) {
                        yield return gizmo;
                    }
                }

                foreach (var gizmo in pawn.mindState.priorityWork.GetGizmos(pawn)) {
                    yield return gizmo;
                }
            }

            if (pawn.drafter != null) {
                foreach (var gizmo in pawn.drafter.InvokeMethod<IEnumerable<Gizmo>>("GetGizmos")) {
                    yield return gizmo;
                }
            }
        }

        public static bool CanTakeOrder(Pawn pawn) => true;
    }
}
