﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;

namespace DraftAnything {
    static class Injector {
        private const BindingFlags bindingFlags = BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance |
                                                    BindingFlags.Static | BindingFlags.GetField | BindingFlags.GetProperty;

        public static void Inject(string originalClassName, string originalMethodName, string injectionClassName, string injectionMethodName) { 
            var replaceType = Type.GetType(originalClassName);
            var injectType = Type.GetType(injectionClassName);

            if (replaceType == null) { throw new Exception(string.Format("Class {0} could not be loaded.", originalClassName)); }
            if (injectType == null) { throw new Exception(string.Format("Class {0} could not be loaded.", injectionClassName)); }

            var replaceMethod = replaceType.GetMethod(originalMethodName, bindingFlags);
            var injectMethod = injectType.GetMethod(injectionMethodName, bindingFlags);

            if (replaceMethod == null) { throw new Exception(string.Format("Method {0} of class {1} could not be loaded.", originalMethodName, originalClassName)); }
            if (injectMethod == null) { throw new Exception(string.Format("Method {0} of class {1} could not be loaded.", injectionMethodName, injectionClassName)); }
            
            // https://ludeon.com/forums/index.php?topic=17143.0
            unsafe
            {
                if (IntPtr.Size == sizeof(Int64)) {
                    // 64-bit systems use 64-bit absolute address and jumps
                    // 12 byte destructive

                    // Get function pointers
                    long Source_Base = replaceMethod.MethodHandle.GetFunctionPointer().ToInt64();
                    long Destination_Base = injectMethod.MethodHandle.GetFunctionPointer().ToInt64();

                    // Native source address
                    byte* Pointer_Raw_Source = (byte*)Source_Base;

                    // Pointer to insert jump address into native code
                    long* Pointer_Raw_Address = (long*)(Pointer_Raw_Source + 0x02);

                    // Insert 64-bit absolute jump into native code (address in rax)
                    // mov rax, immediate64
                    // jmp [rax]
                    *(Pointer_Raw_Source + 0x00) = 0x48;
                    *(Pointer_Raw_Source + 0x01) = 0xB8;
                    *Pointer_Raw_Address = Destination_Base; // ( Pointer_Raw_Source + 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09 )
                    *(Pointer_Raw_Source + 0x0A) = 0xFF;
                    *(Pointer_Raw_Source + 0x0B) = 0xE0;

                } else {
                    // 32-bit systems use 32-bit relative offset and jump
                    // 5 byte destructive

                    // Get function pointers
                    int Source_Base = replaceMethod.MethodHandle.GetFunctionPointer().ToInt32();
                    int Destination_Base = injectMethod.MethodHandle.GetFunctionPointer().ToInt32();

                    // Native source address
                    byte* Pointer_Raw_Source = (byte*)Source_Base;

                    // Pointer to insert jump address into native code
                    int* Pointer_Raw_Address = (int*)(Pointer_Raw_Source + 1);

                    // Jump offset (less instruction size)
                    int offset = (Destination_Base - Source_Base) - 5;

                    // Insert 32-bit relative jump into native code
                    *Pointer_Raw_Source = 0xE9;
                    *Pointer_Raw_Address = offset;
                }
            }
        }
    }
}
